from datetime import datetime
from unittest import TestCase

from pybeamline.bevent import BEvent


class TestBEvent(TestCase):
    def test_get_process_name(self):
        e1 = BEvent("act-a", "case-id")
        self.assertEqual(e1.get_process_name(), "ProcessName")
        e2 = BEvent("act-a", "case-id", process_name="process-name")
        self.assertEqual(e2.get_process_name(), "process-name")

    def test_get_trace_name(self):
        e = BEvent("act-a", "case-id")
        self.assertEqual(e.get_trace_name(), "case-id")

    def test_get_event_name(self):
        e = BEvent("act-a", "case-id")
        self.assertEqual(e.get_event_name(), "act-a")

    def test_get_event_time(self):
        t = datetime.now()
        e = BEvent("act-a", "case-id", event_time=t)
        self.assertEqual(e.get_event_time(), t)



import json
import reactivex
import requests
import hashlib
from pybeamline.bevent import BEvent


def wikimedia_source():
    return reactivex.create(_fetch_stream)


def _fetch_stream(observer, _):
    url = "https://stream.wikimedia.org/v2/stream/recentchange"
    headers = {"Accept": "application/json"}

    with requests.get(url, headers=headers, stream=True) as response:
        if response.status_code == 200:
            for line in response.iter_lines():
                if line:
                    try:
                        data = json.loads(line.decode("utf-8"))
                        e = BEvent(data["type"], hashlib.md5(data["title_url"].encode()).hexdigest(), data["wiki"])
                        attributes_to_copy = ["title", "user", "bot", "comment", "server_url", "server_name", "namespace", "revision", "length", "timestamp", "wiki"]
                        for k in attributes_to_copy:
                            if k in data:
                                e.event_attributes[k] = data[k]
                        observer.on_next(e)
                    except json.JSONDecodeError as e:
                        observer.on_error(e)
        else:
            observer.on_error(Exception(f"Error: {response.status_code}"))

    observer.on_completed()



from datetime import datetime

# These names are from pm4py.objects.log.util.xes which is not imported for performance reasons
DEFAULT_NAME_KEY = 'concept:name'
DEFAULT_TIMESTAMP_KEY = 'time:timestamp'
DEFAULT_TRACEID_KEY = 'concept:name'


class BEvent:
    def __init__(self, activity_name, case_id, process_name="ProcessName", event_time=None):
        self.process_attributes = dict()
        self.trace_attributes = dict()
        self.event_attributes = dict()

        self.process_attributes[DEFAULT_NAME_KEY] = process_name
        self.trace_attributes[DEFAULT_TRACEID_KEY] = case_id
        self.event_attributes[DEFAULT_NAME_KEY] = activity_name
        self.event_attributes[DEFAULT_TIMESTAMP_KEY] = datetime.now() if not event_time else event_time

    def get_process_name(self):
        return self.process_attributes[DEFAULT_NAME_KEY]

    def get_trace_name(self):
        return self.trace_attributes[DEFAULT_TRACEID_KEY]

    def get_event_name(self):
        return self.event_attributes[DEFAULT_NAME_KEY]

    def get_event_time(self):
        return self.event_attributes[DEFAULT_TIMESTAMP_KEY]

    def __str__(self) -> str:
        return "({}, {}, {}, {} - {} - {} - {})".format(
            self.get_event_name(),
            self.get_trace_name(),
            self.get_process_name(),
            str(self.get_event_time()),
            str({c: self.event_attributes[c] for c in self.event_attributes.keys() - {DEFAULT_NAME_KEY, DEFAULT_TIMESTAMP_KEY}}),
            str({c: self.trace_attributes[c] for c in self.trace_attributes.keys() - {DEFAULT_TRACEID_KEY}}),
            str({c: self.process_attributes[c] for c in self.process_attributes.keys() - {DEFAULT_NAME_KEY}})
        )



from reactivex import operators as ops
from reactivex import Observable
from pybeamline.bevent import BEvent
from typing import Callable


def simple_dfg_miner(
        model_update_frequency=10,
        min_relative_frequency=0.75) -> Callable[[Observable[BEvent]], Observable]:

    model_update_frequency = max(model_update_frequency, 2)
    latest_event = dict()  # latest event for each case
    complete_dfg = dict()  # dfg: tuple -> frequency
    observed_events = 0

    def miner(event: BEvent) -> Observable:
        nonlocal observed_events
        nonlocal latest_event
        nonlocal complete_dfg

        activity_name = event.get_event_name()
        case_id = event.get_trace_name()

        if case_id in latest_event:
            relation = (latest_event[case_id], activity_name)
            if relation in complete_dfg:
                complete_dfg[relation] += 1
            else:
                complete_dfg[relation] = 1
        latest_event[case_id] = activity_name

        observed_events += 1
        if observed_events % model_update_frequency == 0 and len(complete_dfg) > 0:
            max_frequency = max(complete_dfg.values())
            if max_frequency > 0:
                m = {k: v/max_frequency for k, v in complete_dfg.items() if v/max_frequency > min_relative_frequency}
                return m
        return None

    return lambda stream: stream.pipe(
        ops.map(miner),
        ops.filter(lambda x: x is not None)
    )



from setuptools import find_packages, setup
from os.path import dirname, join


def read_file(filename):
    with open(join(dirname(__file__), filename)) as f:
        return f.read()


with open("requirements.txt", "r") as f:
    required_packages = f.read().splitlines()
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pybeamline",
    packages=[
        "pybeamline",
        "pybeamline.filters",
        "pybeamline.sources",
        "pybeamline.mappers",
        "pybeamline.algorithms",
        "pybeamline.algorithms.conformance",
        "pybeamline.algorithms.discovery",
    ],
    version="1.0.5",
    description="Python version of Beamline (based on ReactiveX)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Andrea Burattin",
    license="Apache-2.0",
    install_requires=required_packages
)



from pybeamline.sources.xes_log_source import xes_log_source, xes_log_source_from_file
from pybeamline.sources.string_test_source import string_test_source
from pybeamline.sources.mqttxes_source import mqttxes_source
from typing import Union
from pandas import DataFrame
from pm4py.objects.log.obj import EventLog
from reactivex import Observable
from pybeamline.bevent import BEvent


def log_source(log: Union[EventLog, DataFrame, list, str]) -> Observable[BEvent]:
    if type(log) is EventLog or type(log) is DataFrame:
        return xes_log_source(log)
    if type(log) is list and len(log) > 1:
        return string_test_source(log)
    if type(log) is str:
        return xes_log_source_from_file(log)
